;=====================================================
;DISPLAYS MAP ON SCREEN (BACK BUFFER) WHEN CALLED BY EXECUTABLE
;=====================================================
;* does not perform any error-checking as it would have no use
;[ebp-4] = loaded blinker bitmap handle
;[ebp-8] = compatible DC (as temp buffer) of DX surface
;[ebp-12] = Y value of map blinker location
;[ebp-16] = X value of map blinker location
;edi = DX surface Device Context
;ebx = used to test if position is in map

SHOWMAP_CODE:
push ebp
mov ebp,esp
sub esp,16
push ebx
push esi
push edi
push ecx
mov esi,[BaseData]

;------- Calculate blinker position in map
GETPLAYERLOCADDR	;eax points to x,y,z positions

fld DWORD [eax]
fsub [anchorXvalue]	;substract top-corner X-anchor from Xposition
fdiv [scaleXvalue]	;Xfloatdisplacement / (X float units/pixel) = # pixels (Xpos)
fistp DWORD [ebp-16]	;convert to integer and store

fld DWORD [eax+4]
fsub [anchorYvalue]	;substract top-corner Y-anchor from Xposition
fdiv [scaleYvalue]	;Yfloatdisplacement / (Y float units/pixel) = # pixels (Ypos)
fistp DWORD [ebp-12]	;convert to integer and store

;------- test position and clip blinker to known map edges
mov ebx,SRCPAINT

mov eax,[ebp-16]       ;outside map (to the left)?
test eax,eax
jns BlinkerPosXPositive_SHOWMAP
xor eax,eax
mov ebx,SRCAND
BlinkerPosXPositive_SHOWMAP:

cmp eax,[bitmapwidth]  ;outside map (to the right)?
jb BlinkerInMapX_SHOWMAP
mov eax,[bitmapwidth]
dec eax
mov ebx,SRCAND
BlinkerInMapX_SHOWMAP:

;sub eax,6      ;convert location point from center of blinker to corner
mov [ebp-16],eax	;EDIT: no longer needed -> map displaced instead

mov eax,[ebp-12]       ;outside map (upward)?
test eax,eax
jns BlinkerPosYPositive_SHOWMAP
xor eax,eax
mov ebx,SRCAND
BlinkerPosYPositive_SHOWMAP:

cmp eax,[bitmapheight] ;outside map (downward)?
jb BlinkerInMapY_SHOWMAP
mov eax,[bitmapheight]
dec eax
mov ebx,SRCAND
BlinkerInMapY_SHOWMAP:

;sub eax,6
mov [ebp-12],eax

;------- get main DirectX surface from EXE

	mov eax,[BaseDDSurfaceADDR]
	mov ecx,[eax]
	call [EXEPROC_GETDDSURFDC]
	mov edi,eax

	cmp [currentbitmap],0
	jnz CurrentBitmapValid_SHOWMAP
invoke TextOut,eax,0,0,_nocurrentbitmap,(_nocurrentbitmap_end - _nocurrentbitmap)
	jmp NOCURRENTBITMAP_SHOWMAP
	CurrentBitmapValid_SHOWMAP:
;------------------- create alternate DC for blinker blitting
invoke LoadBitmap,[DLLHandle],100
       mov [ebp-4],eax

invoke CreateCompatibleDC,edi	    ;DDsurface DC
       mov [ebp-8],eax

invoke SelectObject,eax,DWORD [ebp-4]

;-------------------- draws bitmap directly to screen (no real need
;                       for double-buffering
       mov ecx,[currentbitmap]
       add ecx,14		 ;BITMAPINFO start
       mov eax,[ecx-4]
       add eax,[currentbitmap]	 ;DIB bits start
invoke SetDIBitsToDevice,edi,6,6,[bitmapwidth],\
	[bitmapheight],0,0,0,[bitmapheight],eax,ecx,DIB_RGB_COLORS

;--------------------- draws 'blinker' to screen (over map)
       test [Show_Blinker],1
       jz NoBlinkerDraw_SHOWMAP

invoke BitBlt,edi,DWORD [ebp-16],DWORD [ebp-12],\
	13,13,DWORD [ebp-8],0,0,ebx
NoBlinkerDraw_SHOWMAP:
;------------------------ calculate whether to display blinker next

invoke GetTickCount
	mov edx,eax

	mov ecx,[LastTickCount]
	sub eax,ecx
	cmp eax,DotBlinkSpeed
	jb NoBlinkStateChange_SHOWMAP
	mov [LastTickCount],edx
	xor BYTE [Show_Blinker],1

NoBlinkStateChange_SHOWMAP:
;------------------------ exit code

invoke DeleteDC, DWORD [ebp-8] ;delete compatible DC

invoke DeleteObject,DWORD [ebp-4]

NOCURRENTBITMAP_SHOWMAP:
	mov eax,[BaseDDSurfaceADDR]
	push edi		 ;DDraw ReleaseDC
	mov ecx,[eax]
	call [EXEPROC_RELEASEDDSURFDC]

NoDDSurface_SHOWMAP:
pop ecx
pop edi
pop esi
pop ebx
mov esp,ebp
pop ebp
ret
